class TeamXError(Exception):
    """Base exception for TeamXBot errors."""
    pass

class TokenInvalidError(TeamXError):
    """Raised when the bot token is invalid or empty."""
    pass

class APIError(TeamXError):
    """Raised when Telegram API returns an error."""
    pass

class HandlerError(TeamXError):
    """Raised when a handler fails."""
    pass

class BotRunningError(TeamXError):
    """Raised when trying to start a bot that is already running."""
    pass

class FileNotFoundError(TeamXError):
    """Raised when a file is not found."""
    pass


import aiohttp
import asyncio
import logging
from typing import List, Dict
from .handlers import Handler
from .errors import TokenInvalidError, BotRunningError, APIError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("TeamXBot")

class TeamXBot:
    _instance = None

    def __init__(self, token: str, parse_mode: str = "Markdown"):
        if not token:
            raise TokenInvalidError("Bot token cannot be empty")
        if TeamXBot._instance is not None:
            raise BotRunningError("A bot instance is already running")
        TeamXBot._instance = self
        self.token = token
        self.base_url = f"https://api.telegram.org/bot{token}/"
        self.handlers: List[Handler] = []
        self.parse_mode = parse_mode
        self.session = None
        self.is_running = False
        logger.info("TeamXBot initialized")

    def register_handler(self, handler: Handler):
        try:
            handler.bot = self
            self.handlers.append(handler)
            logger.debug(f"Handler registered: {handler.type} - {handler.pattern}")
        except Exception as e:
            logger.error(f"Error registering handler: {e}")
            raise HandlerError(f"Failed to register handler: {e}")

    async def _request(self, method: str, data: Dict = None) -> Dict:
        try:
            if not self.session:
                self.session = aiohttp.ClientSession()
            url = self.base_url + method
            async with self.session.post(url, json=data) as response:
                result = await response.json()
                if not result.get("ok"):
                    raise APIError(f"API error in {method}: {result.get('description')}")
                return result
        except Exception as e:
            logger.error(f"Request error in {method}: {e}")
            raise APIError(f"Failed to execute {method}: {e}")

    async def send_message(self, chat_id: int, text: str, reply_markup: Dict = None):
        try:
            data = {"chat_id": chat_id, "text": text, "parse_mode": self.parse_mode}
            if reply_markup:
                data["reply_markup"] = reply_markup
            return await self._request("sendMessage", data)
        except Exception as e:
            logger.error(f"Error sending message: {e}")
            raise APIError(f"Failed to send message: {e}")

    async def get_updates(self, offset: int = None) -> List[Dict]:
        try:
            data = {"timeout": 30, "offset": offset}
            result = await self._request("getUpdates", data)
            return result.get("result", [])
        except Exception as e:
            logger.error(f"Error getting updates: {e}")
            return []

    async def start_polling(self):
        if self.is_running:
            raise BotRunningError("Bot is already polling")
        self.is_running = True
        offset = None
        logger.info("Started polling...")
        try:
            while self.is_running:
                updates = await self.get_updates(offset)
                for update in updates:
                    offset = update["update_id"] + 1
                    await self._process_update(update)
                await asyncio.sleep(0.1)  # Avoid CPU overload
        except Exception as e:
            logger.error(f"Polling error: {e}")
            raise
        finally:
            self.is_running = False

    async def _process_update(self, update: Dict):
        try:
            for handler in self.handlers:
                if handler.matches(update):
                    await handler.execute(update)
        except Exception as e:
            logger.error(f"Error processing update: {e}")

    def run(self):
        try:
            import inspect
            caller_frame = inspect.currentframe().f_back
            for name, obj in caller_frame.f_globals.items():
                if hasattr(obj, 'handler'):
                    self.register_handler(obj.handler)
            asyncio.run(self.start_polling())
        except Exception as e:
            logger.error(f"Error starting bot: {e}")
            raise


from typing import Callable, Dict
from functools import wraps
import logging
from .errors import HandlerError

logger = logging.getLogger("TeamXBot.Handlers")

class Handler:
    def __init__(self, type_: str, pattern: str, callback: Callable):
        self.type = type_
        self.pattern = pattern
        self.callback = callback
        self.bot = None

    def matches(self, update: Dict) -> bool:
        try:
            if self.type == "command":
                message = update.get("message", {}).get("text", "")
                return message.startswith(f"/{self.pattern}")
            return False
        except Exception as e:
            logger.error(f"Error in handler matching: {e}")
            return False

    async def execute(self, update: Dict):
        try:
            if not self.bot:
                raise HandlerError("Bot instance not set for handler")
            await self.callback(self.bot, update)
        except Exception as e:
            logger.error(f"Error executing handler {self.pattern}: {e}")
            raise HandlerError(f"Handler execution failed: {e}")

def command(name: str):
    def decorator(func: Callable):
        @wraps(func)
        async def wrapper(bot, update):
            try:
                await func(bot, update)
            except Exception as e:
                logger.error(f"Error in command '{name}': {e}")
                raise HandlerError(f"Command {name} failed: {e}")
        wrapper.handler = Handler("command", name, wrapper)
        logger.debug(f"Registered command: {name}")
        return wrapper
    return decorator


from .bot import TeamXBot
from .handlers import command
from .inline import InlineButton, InlineKeyboard
from .media import Media
from .keyboard import ReplyButton, ReplyKeyboard
from .errors import TeamXError, TokenInvalidError, APIError, HandlerError, BotRunningError
from .plugins import Plugin, load_plugin

__version__ = "0.1.0"


from typing import Dict
import aiohttp
import logging
from .errors import APIError, TeamXError

logger = logging.getLogger("TeamXBot.Media")

class Media:
    def __init__(self, bot):
        self.bot = bot

    async def send_photo(self, chat_id: int, photo: str, caption: str = None):
        try:
            data = {"chat_id": chat_id, "photo": photo}
            if caption:
                data["caption"] = caption
                data["parse_mode"] = self.bot.parse_mode
            result = await self.bot._request("sendPhoto", data)
            logger.info(f"Sent photo to chat {chat_id}")
            return result
        except Exception as e:
            logger.error(f"Error sending photo: {e}")
            raise APIError(f"Failed to send photo: {e}")

    async def send_video(self, chat_id: int, video: str, caption: str = None):
        try:
            data = {"chat_id": chat_id, "video": video}
            if caption:
                data["caption"] = caption
                data["parse_mode"] = self.bot.parse_mode
            result = await self.bot._request("sendVideo", data)
            logger.info(f"Sent video to chat {chat_id}")
            return result
        except Exception as e:
            logger.error(f"Error sending video: {e}")
            raise APIError(f"Failed to send video: {e}")

    async def send_audio(self, chat_id: int, audio: str, caption: str = None):
        try:
            data = {"chat_id": chat_id, "audio": audio}
            if caption:
                data["caption"] = caption
                data["parse_mode"] = self.bot.parse_mode
            result = await self.bot._request("sendAudio", data)
            logger.info(f"Sent audio to chat {chat_id}")
            return result
        except Exception as e:
            logger.error(f"Error sending audio: {e}")
            raise APIError(f"Failed to send audio: {e}")

    async def send_document(self, chat_id: int, document: str, caption: str = None):
        try:
            data = {"chat_id": chat_id, "document": document}
            if caption:
                data["caption"] = caption
                data["parse_mode"] = self.bot.parse_mode
            result = await self.bot._request("sendDocument", data)
            logger.info(f"Sent document to chat {chat_id}")
            return result
        except Exception as e:
            logger.error(f"Error sending document: {e}")
            raise APIError(f"Failed to send document: {e}")

    async def send_file(self, chat_id: int, file_path: str, media_type: str, caption: str = None):
        try:
            async with aiohttp.ClientSession() as session:
                with open(file_path, "rb") as f:
                    data = aiohttp.FormData()
                    data.add_field("chat_id", str(chat_id))
                    data.add_field(media_type, f, filename=file_path)
                    if caption:
                        data.add_field("caption", caption)
                        data.add_field("parse_mode", self.bot.parse_mode)
                    async with session.post(f"{self.bot.base_url}send{media_type.capitalize()}", data=data) as response:
                        result = await response.json()
                        if not result.get("ok"):
                            raise APIError(f"Failed to send {media_type}: {result.get('description')}")
                        logger.info(f"Sent {media_type} file to chat {chat_id}")
                        return result
        except FileNotFoundError:
            logger.error(f"File not found: {file_path}")
            raise TeamXError(f"File {file_path} not found")
        except Exception as e:
            logger.error(f"Error sending file {file_path}: {e}")
            raise APIError(f"Failed to send file: {e}")


import click
import os
import logging
from .errors import TeamXError

logger = logging.getLogger("TeamXBot.CLI")

@click.group()
def cli():
    """TeamXBot CLI for creating and managing bots."""
    pass

@cli.command()
@click.argument("name")
def create(name: str):
    """Create a new bot project with the given name."""
    try:
        os.makedirs(name)
        with open(f"{name}/bot.py", "w") as f:
            f.write("""
from team_x_og import TeamXBot, command

bot = TeamXBot("YOUR_BOT_TOKEN")

@command("start")
async def start_command(bot, update):
    chat_id = update["message"]["chat"]["id"]
    await bot.send_message(chat_id, "Hello, welcome to my bot!")

if __name__ == "__main__":
    bot.run()
            """)
        with open(f"{name}/requirements.txt", "w") as f:
            f.write("team_x_og>=0.1.0\n")
        logger.info(f"Created bot project: {name}")
        click.echo(f"Created bot project: {name}")
        click.echo(f"Run: cd {name} && pip install -r requirements.txt && python bot.py")
    except Exception as e:
        logger.error(f"Error creating project: {e}")
        click.echo(f"Error creating project: {e}")

if __name__ == "__main__":
    cli()


from typing import List, Dict
import logging
from .errors import TeamXError

logger = logging.getLogger("TeamXBot.Keyboard")

class ReplyButton:
    def __init__(self, text: str):
        if not text:
            raise TeamXError("Button text cannot be empty")
        self.text = text
        logger.debug(f"Created reply button: {text}")

    def to_dict(self) -> Dict:
        return {"text": self.text}

class ReplyKeyboard:
    def __init__(self, resize: bool = True, one_time: bool = False):
        self.buttons: List[List[ReplyButton]] = []
        self.resize = resize
        self.one_time = one_time

    def add_row(self, *buttons: ReplyButton):
        if not buttons:
            raise TeamXError("At least one button required in row")
        self.buttons.append([btn.to_dict() for btn in buttons])
        logger.debug("Added row to reply keyboard")

    def to_dict(self) -> Dict:
        return {
            "keyboard": self.buttons,
            "resize_keyboard": self.resize,
            "one_time_keyboard": self.one_time
        }


import logging
from typing import Callable
from .handlers import Handler
from .errors import TeamXError

logger = logging.getLogger("TeamXBot.Plugins")

class Plugin:
    def __init__(self, name: str):
        self.name = name
        self.handlers = []
        logger.debug(f"Created plugin: {name}")

    def register_handler(self, handler: Handler):
        self.handlers.append(handler)
        logger.debug(f"Plugin {self.name} registered handler: {handler.type}")

def load_plugin(bot, plugin_path: str):
    try:
        import importlib.util
        import sys
        spec = importlib.util.spec_from_file_location("plugin", plugin_path)
        if not spec:
            raise TeamXError(f"Invalid plugin path: {plugin_path}")
        module = importlib.util.module_from_spec(spec)
        sys.modules["plugin"] = module
        spec.loader.exec_module(module)
        plugin = getattr(module, "plugin", None)
        if plugin:
            for handler in plugin.handlers:
                bot.register_handler(handler)
            logger.info(f"Loaded plugin: {plugin.name}")
        else:
            raise TeamXError(f"No plugin defined in {plugin_path}")
    except Exception as e:
        logger.error(f"Error loading plugin {plugin_path}: {e}")
        raise TeamXError(f"Failed to load plugin: {e}")


from typing import List, Dict
import logging
from .errors import TeamXError

logger = logging.getLogger("TeamXBot.Inline")

class InlineButton:
    def __init__(self, text: str, callback_data: str = None, url: str = None):
        if not text:
            raise TeamXError("Button text cannot be empty")
        if callback_data and url:
            raise TeamXError("Button cannot have both callback_data and url")
        self.text = text
        self.callback_data = callback_data
        self.url = url
        logger.debug(f"Created button: {text}")

    def to_dict(self) -> Dict:
        result = {"text": self.text}
        if self.callback_data:
            result["callback_data"] = self.callback_data
        if self.url:
            result["url"] = self.url
        return result

class InlineKeyboard:
    def __init__(self):
        self.buttons: List[List[InlineButton]] = []

    def add_row(self, *buttons: InlineButton):
        if not buttons:
            raise TeamXError("At least one button required in row")
        self.buttons.append([btn.to_dict() for btn in buttons])
        logger.debug("Added row to inline keyboard")

    def to_dict(self) -> Dict:
        return {"inline_keyboard": self.buttons}


from setuptools import setup, find_packages

setup(
    name="team_x_og",
    version="0.1.0",
    packages=find_packages(),
    install_requires=["aiohttp>=3.8.0", "click>=8.0.0"],
    author="Your Name",
    author_email="armanhacker95@gmail.com",
    description="A powerful and beginner-friendly Telegram bot library",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/ARMAN01011",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    entry_points={
        "console_scripts": [
            "teamxbot=team_x_og.cli:cli",
        ]
    }
)


import unittest
from team_x_og import TeamXBot, TokenInvalidError, BotRunningError

class TestTeamXBot(unittest.TestCase):
    def test_empty_token(self):
        with self.assertRaises(TokenInvalidError):
            TeamXBot("")

    def test_multiple_instances(self):
        bot1 = TeamXBot("DUMMY_TOKEN")
        with self.assertRaises(BotRunningError):
            TeamXBot("ANOTHER_TOKEN")

if __name__ == "__main__":
    unittest.main()